@@ -3,15 +3,18 @@ use warnings;
 
 package JMAP::Tester;
 # ABSTRACT: a JMAP client made for testing JMAP servers
-$JMAP::Tester::VERSION = '0.006';
+$JMAP::Tester::VERSION = '0.007';
 use Moo;
 
 use Encode qw(encode_utf8);
 use HTTP::Request;
+use JMAP::Tester::Logger::Null;
 use JMAP::Tester::Response;
 use JMAP::Tester::Result::Auth;
+use JMAP::Tester::Result::Download;
 use JMAP::Tester::Result::Failure;
 use JMAP::Tester::Result::Upload;
+use Module::Runtime ();
 use Safe::Isa;
 use URI;
 
@@ -93,14 +96,6 @@ has _json_typist => (
 for my $type (qw(api authentication download upload)) {
   has "$type\_uri" => (
     is => 'rw',
-    isa => sub {
-      die "provided $type\_uri is not a URI object" unless $_[0]->$_isa('URI');
-    },
-    coerce => sub {
-      return $_[0] if $_[0]->$_isa('URI');
-      die "can't use reference as a URI" if ref $_[0];
-      URI->new($_[0]);
-    },
     predicate => "has_$type\_uri",
     clearer   => "clear_$type\_uri",
   );
@@ -110,6 +105,8 @@ has ua => (
   is   => 'ro',
   lazy => 1,
   default => sub {
+    my ($self) = @_;
+
     require LWP::UserAgent;
     my $ua = LWP::UserAgent->new;
     $ua->cookie_jar({});
@@ -124,13 +121,19 @@ has ua => (
 
 sub _set_cookie {
   my ($self, $name, $value) = @_;
+
+  Carp::confess("can't _set_cookie without api_uri configured")
+    unless $self->has_api_uri;
+
+  my $uri = URI->new($self->api_uri);
+
   $self->ua->cookie_jar->set_cookie(
     1,
     $name,
     $value,
     '/',
-    $self->api_uri->host,
-    $self->api_uri->port,
+    $uri->host,
+    $uri->port,
     0,
     0,
     86400,
@@ -236,16 +239,30 @@ sub request {
   my $json = $self->json_encode(\@suffixed);
 
   my $post = HTTP::Request->new(
-    POST => $self->api_uri->as_string,
+    POST => $self->api_uri,
     [
       'Content-Type' => 'application/json',
     ],
     $json,
   );
 
+  $self->ua->set_my_handler(request_send => sub {
+    my ($req) = @_;
+    $self->_logger->log_jmap_request({
+      jmap_array   => \@suffixed,
+      json         => $json,
+      http_request => $req,
+    });
+    return;
+  });
+
   my $http_res = $self->ua->request($post);
 
   unless ($http_res->is_success) {
+    $self->_logger->log_jmap_response({
+      http_response => $http_res,
+    });
+
     return JMAP::Tester::Result::Failure->new({
       http_response => $http_res,
     });
@@ -257,6 +274,12 @@ sub request {
     $self->json_decode( $http_res->decoded_content )
   );
 
+  $self->_logger->log_jmap_response({
+    jmap_array    => $data,
+    json          => $http_res->decoded_content,
+    http_response => $http_res,
+  });
+
   return JMAP::Tester::Response->new({
     struct => $data,
     _json_typist  => $self->_json_typist,
@@ -264,6 +287,23 @@ sub request {
   });
 }
 
+has _logger => (
+  is => 'ro',
+  default => sub {
+    if ($ENV{JMAP_TESTER_LOGGER}) {
+      my ($class, $filename) = split /:/, $ENV{JMAP_TESTER_LOGGER};
+      $class = "JMAP::Tester::Logger::$class";
+      Module::Runtime::require_module($class);
+
+      return $class->new({
+        writer => $filename // 'jmap-tester-{T}-{PID}.log'
+      });
+    }
+
+    JMAP::Tester::Logger::Null->new({ writer => \undef });
+  },
+);
+
 #pod =method upload
 #pod
 #pod   my $result = $tester->upload($mime_type, $blob_ref, \%arg);
@@ -283,23 +323,46 @@ sub upload {
   Carp::confess("can't upload without upload_uri")
     unless $self->upload_uri;
 
-  my $res = $self->ua->post(
-    $self->upload_uri,
-    Content => $$blob_ref,
-    'Content-Type' => $mime_type,
+  my $post = HTTP::Request->new(
+    POST => $self->upload_uri,
+    [
+      'Content-Type' => $mime_type,
+    ],
+    $$blob_ref,
   );
 
+  $self->ua->set_my_handler(request_send => sub {
+    my ($req) = @_;
+    $self->_logger->log_upload_request({
+      http_request => $req,
+    });
+    return;
+  });
+
+  my $res = $self->ua->request($post);
+
   unless ($res->is_success) {
+    $self->_logger->log_upload_response({
+      http_response => $res,
+    });
+
     return JMAP::Tester::Result::Failure->new({
       http_response => $res,
     });
   }
 
+  my $json = $res->decoded_content;
+  my $blob = $self->apply_json_types( $self->json_decode( $json ) );
+
+  $self->_logger->log_upload_response({
+    json          => $json,
+    blob_struct   => $blob,
+    http_response => $res,
+  });
+
   return JMAP::Tester::Result::Upload->new({
     http_response => $res,
-    payload       => $self->apply_json_types(
-      $self->json_decode( $res->decoded_content )
-    ),
+    payload       => $blob,
   });
 }
 
@@ -309,10 +372,12 @@ sub upload {
 #pod
 #pod Valid arguments are:
 #pod
-#pod   blobId    - the blob to download (required)
-#pod   accountId - the account for which we're downloading (required)
+#pod   blobId    - the blob to download (no default)
+#pod   accountId - the account for which we're downloading (no default)
 #pod   name      - the name we want the server to provide back (default: "download")
 #pod
+#pod If the download URI template has a C<blobId> or C<accountId> placeholder but no
+#pod argument for that is given to C<download>, an exception will be thrown.
 #pod
 #pod The return value will either be a L<failure
 #pod object|JMAP::Tester::Result::Failure> or an L<upload
@@ -320,24 +385,40 @@ sub upload {
 #pod
 #pod =cut
 
+my %DL_DEFAULT = (name => 'download');
+
 sub download {
   my ($self, $arg) = @_;
 
-  Carp::confess("can't download without download") unless $self->download_uri;
+  Carp::confess("can't download without download_uri")
+    unless my $uri = $self->download_uri;
 
-  for my $name (qw(blobId accountId)) {
-    Carp::confess("missing required parameter $name")
-      unless defined $arg->{$name};
+  for my $param (qw(blobId accountId name)) {
+    next unless $uri =~ /\{$param\}/;
+    my $value = $arg->{ $param } // $DL_DEFAULT{ $param };
+
+    Carp::confess("missing required template parameter $param")
+      unless defined $value;
+
+    $uri =~ s/\{$param\}/$value/g;
   }
 
-  my $uri = $self->download_uri;
-  for my $var (qw(blobId accountId name)) {
-    my $value = $arg->{$var} // $var; # only "name" could be undef
-    $uri =~ s/\{$var\}/$arg->{$var}/g;
-  }
+  my $get = HTTP::Request->new(GET => $uri);
+
+  $self->ua->set_my_handler(request_send => sub {
+    my ($req) = @_;
+    $self->_logger->log_download_request({
+      http_request => $req,
+    });
+    return;
+  });
 
   my $res = $self->ua->get($uri);
 
+  $self->_logger->log_download_response({
+    http_response => $res,
+  });
+
   unless ($res->is_success) {
     return JMAP::Tester::Result::Failure->new({
       http_response => $res,
@@ -370,12 +451,13 @@ sub simple_auth {
     deviceName    => 'JMAP Testing Client',
   });
 
-  my $start_res = HTTP::Request->new(
-    POST => $self->authentication_uri->as_string,
+  my $start_res = $self->ua->post(
+    $self->authentication_uri->as_string,
     [
-      'Content-Type' => 'application/json',
+      'Content-Type' => 'application/json; charset=utf-8',
+      'Accept'       => 'application/json',
+      'Content'      => $start_json,
     ],
-    $start_json,
   );
 
   unless ($start_res->code == 200) {
@@ -400,12 +482,13 @@ sub simple_auth {
     value   => $password,
   });
 
-  my $next_res = HTTP::Request->new(
-    POST => $self->authentication_uri->as_string,
+  my $next_res = $self->ua-post(
+    $self->authentication_uri->as_string,
     [
-      'Content-Type' => 'application/json',
+      'Content-Type' => 'application/json; charset=utf-8',
+      'Accept'       => 'application/json',
+      'Content'      => $next_json,
     ],
-    $next_json,
   );
 
   unless ($next_res->code == 201) {
@@ -469,7 +552,7 @@ JMAP::Tester - a JMAP client made for testing JMAP servers
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 OVERVIEW
 
@@ -585,10 +668,13 @@ result|JMAP::Tester::Result::Upload>.
 
 Valid arguments are:
 
-  blobId    - the blob to download (required)
-  accountId - the account for which we're downloading (required)
+  blobId    - the blob to download (no default)
+  accountId - the account for which we're downloading (no default)
   name      - the name we want the server to provide back (default: "download")
 
+If the download URI template has a C<blobId> or C<accountId> placeholder but no
+argument for that is given to C<download>, an exception will be thrown.
+
 The return value will either be a L<failure
 object|JMAP::Tester::Result::Failure> or an L<upload
 result|JMAP::Tester::Result::Download>.
