@@ -1,7 +1,7 @@
 use v5.10.0;
 package JMAP::Tester::Response::Sentence::Set;
 # ABSTRACT: the kind of sentence you get in reply to a setFoos call
-$JMAP::Tester::Response::Sentence::Set::VERSION = '0.006';
+$JMAP::Tester::Response::Sentence::Set::VERSION = '0.007';
 use Moo;
 
 #pod =head1 OVERVIEW
@@ -99,7 +99,29 @@ sub created_ids {
   map {; $_->{id} } values %{ $_[0]->created }
 }
 
-sub updated_ids   { @{ $_[0]{arguments}{updated} } }
+sub updated_ids   {
+  my ($self) = @_;
+  my $updated = $_[0]{arguments}{updated} // {};
+
+  if (ref $updated eq 'ARRAY') {
+    return @$updated;
+  }
+
+  return keys %$updated;
+}
+
+sub updated {
+  my ($self) = @_;
+
+  my $updated = $_[0]{arguments}{updated} // {};
+
+  if (ref $updated eq 'ARRAY') {
+    return { map {; $_ => undef } @$updated };
+  }
+
+  return $updated;
+}
+
 sub destroyed_ids { @{ $_[0]{arguments}{destroyed} } }
 
 # Is this the best API to provide?  I dunno, maybe.  Usage will tell us whether
@@ -126,7 +148,7 @@ JMAP::Tester::Response::Sentence::Set - the kind of sentence you get in reply to
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 OVERVIEW
 
