@@ -3,7 +3,7 @@ use strict;
 
 package JMAP::Tester::Result::Download;
 # ABSTRACT: what you get when you download a blob
-$JMAP::Tester::Result::Download::VERSION = '0.006';
+$JMAP::Tester::Result::Download::VERSION = '0.007';
 use Moo;
 with 'JMAP::Tester::Role::Result';
 
@@ -19,8 +19,8 @@ sub is_success { 1 }
 has bytes_ref => (
   is   => 'ro',
   lazy => 1,
-  default => {
-    my $str = $self->http_response->decoded_content(charset => 'none');
+  default => sub {
+    my $str = $_[0]->http_response->decoded_content(charset => 'none');
     return $str;
   },
 );
@@ -39,7 +39,7 @@ JMAP::Tester::Result::Download - what you get when you download a blob
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 OVERVIEW
 
